/*
 * Copyright (C) 2015  Franois GUILLET
 * 
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 * auteur : Franois GUILLET
 * date : janvier 2015
 * utilit :
 * Cette classe permet de grer des fichiers de configuration. 
 * On l'utilise pour obtenir facilement les valeurs associes
 * aux cls du fichier (voir ci-après l'exemple de fichier de configuration).
 * Pratiquement cela s'utilise de la manière suivante :
 * 
 *	int cle1;
 *	float cle2;
 *	string cle3;
 *	long cle8;
 * 
 *	Configuration *config = new Configuration("config.conf");
 *	if (config->estValide())
 *	{
 *		if (config->isPresent("cl4"))
 *			cout << "cl4 est prsente" << endl;
 *		else
 *			cout << "cl4 est absente" << endl;
 *		
 *		// obtension d'une valeur entière
 *		if (config->isPresent("cl1"))
 *			cle1 = config->getIntVal("cl1");
 *		
 *		// obtension d'une valeur flottante
 *		if (config->isPresent("cl1"))
 *			cle1 = config->getIntVal("cl1");
 *		
 *		// obtension d'un string 
 *		if (config->isPresent("cl1"))
 *			cle1 = config->getIntVal("cl1");
 *		
 *		// obtension d'un long
 *		if (config->isPresent("cl1"))
 *			cle1 = config->getIntVal("cl1");
 }
 delete config;
 
 *
 * exemple de fichier de configuration :
 * 
 *  
 *	#commentaire...
 *	;commentaire...
 *	; commentaire...
 *	cl1=	65
 *	cl2		=	65.00
 *	
 *	#commentaire.#..
 *	cl3  = franois GUILLET
 *	cl4
 *	cl5
 *	cl6 = "un exemple de valeur6 :  = #    "
 *	cl7 = #;#
 *	
 *	cl8 = 20144458746695
 ***/

#ifndef Configuration_h
#define Configuration_h
#include <cstdlib>
#include <map>
#include <iostream>
#include <string>
using namespace std;

class Configuration
{
	
public:
	// constructeur : ficherConfig est l'adresse du fichier
	// de configuration  analyser
	Configuration(string ficherConfig);
	~Configuration();
	
	// renvoi vrai si la configuration a t lue
	// (si la map a t remplie avec au moins une valeur)
	bool estValide();	
	// renvoi la valeur associe  la cl sous forme d'entier
	int getIntVal(string cle);
	// renvoi la valeur associe  la cl sous forme d'entier
	long getLongVal(string cle);
	// renvoi la valeur associe  la cl sous forme de flottant
	float getFloatVal(string cle);
	// renvoi la valeur associe  la cl sous forme de string
	string getStringVal(string cle);
	// renvoi vrai si la cl est prsente
	bool isPresent(string cle);
	
private:
	
	// map de la configuration
	map<string, string> config;
	
	// vrai si la configuration a t lue
	// (si la map a t remplie avec au moins une valeur)
	bool valide;
	
	// mthode pratiques lors de l'initialisation de la map :
	// renvoi vrai en cas de commentaire
	bool isComment(string ligne);
	// renvoie la cl
	string getCle(string ligne);
	// renvoie la valeur
	string getVal(string ligne);
	
};

#endif
