#include "Gpio.h"
#include <string>
#include <iostream>
using namespace std;


/// constructeur : 
/// permet de choisir quel port sera utilisé
/// le numéro de port doit être compris entre 1 et 28
Gpio::Gpio(int numeroPort)
{
  numPort = numeroPort;
}

/// permet de choisir la direction d'utilisation du gpio (entrée ou sortie)
/// il sera créé un dossier dans "/sys/class/gpio/gpio{numeroPort}
/// si utilisationEnEntree vaut false, cela indique que le port sera utilisé en sortie
/// si utilisationEnEntree vaut true, cela indique que le port sera utilisé en entrée
void Gpio::configure(bool utilisationEnEntree)
{
  string chemin;
  char* carac = new char[3];
  ofstream exportgpio; 
  
  sprintf(carac, "%i", numPort);
  string strNumPin = string(carac);
  
  delete carac;
  
  //Création du répertoire. Peut importe s'il existe déjà ?
  chemin = "/sys/class/gpio/export";
  exportgpio.open(chemin.c_str(), std::ofstream::out | std::ofstream::app);
  exportgpio << numPort;
  exportgpio.close();
  
  //   paramétrage de la direction
  chemin = "/sys/class/gpio/gpio" + strNumPin + "/direction";
  exportgpio.open(chemin.c_str(), std::ofstream::out | std::ofstream::app);
  
  if (utilisationEnEntree)
  {
    exportgpio << "in";
  }
  else
  {
    exportgpio << "out";
  }
  exportgpio.close();
  
}

/// destructeur virtuel
Gpio::~Gpio()
{
  
}

/// attention, le fait d'appeler cette méthode supprime le
/// dossier créé par la méthode configure, et cela ramène la valeur par défaut (si on est en sortie) du gpio matériel
void Gpio::supprimerPort()
{
  string chemin;
  ofstream unexportgpio; 
  chemin = "/sys/class/gpio/unexport";
  unexportgpio.open (chemin.c_str(), std::ofstream::out | std::ofstream::app);
  unexportgpio << numPort;
  unexportgpio.close();
}

/// permet d'écrire la valeur en sortie
/// si value vaut true, la valeur du port matériel sera mise à 3.3V
/// si value vaut false, la valeur du port matériel sera mise à 0V
void Gpio::changerValeur(bool value)
{
  string chemin;
  string strNumPin;
  
  char* strC = new char[10];
  sprintf(strC, "%i", numPort);
  strNumPin = string(strC);
  delete strC;
  
  ofstream portGpio; 
  
  chemin = "/sys/class/gpio/gpio" + strNumPin + "/value";
  portGpio.open (chemin.c_str(), std::ofstream::out | std::ofstream::app);
  if (value)
     portGpio << "1";
  else
    portGpio << "0";
  portGpio.close();
}

/// permet de lire la valeur en sortie
/// si value vaut true, cela signifie que la valeur du port matériel est connectée à 3.3V
/// si value vaut false, cela signifie que la valeur du port matériel est connectée à 0V
bool Gpio::valeurSortie()
{
/*
  //   paramétrage de la direction
  chemin = "/sys/class/gpio/gpio" + numPin + "/direction";
  exportgpio.open (chemin.c_str(), std::ofstream::out);
  exportgpio << "in";
  exportgpio.close();
  */
  // lecture de la valeur de sortie
  
  string strNumPin;
  
  char* strC = new char[10];
  sprintf(strC, "%i", numPort);
  strNumPin = string(strC);
  delete strC;
  
  string chemin;
  ifstream importgpio; 
  chemin = "/sys/class/gpio/gpio" + strNumPin + "/value";
  importgpio.open (chemin.c_str(), std::ofstream::out);
  // (importgpio peut être vu comme un string (???))
  string val;
  importgpio >> val;
  importgpio.close();
  if (val == "1")
    return true;
  else
    return false;
}
