/*
 * 
 * // Classe est utile pour gérer un port GPIO sur un ordinnateur de la famille des Raspberry.
 * 
 * // Cette classe n'est pas finie : elle ne gère pas les gpio en entrée
 * 
 * // Les ports d'un gpio de raspberry sont numéroté de 1 à 28.
 * // Pour utiliser cette classe, il faut procéder de la manière suivante :
 * 
 * // initialiser l'objet en spécifiant le port qui sera utilisé (il peut être déjà configuré dans /sys/classe/gpio)
 * Gpio* gpio = new Gpio(numPort);
 * // configurer le port et spécifiant s'il sera utilisé en entrée ou en sortie
 * gpio->configure(false);	// utilisation en sortie
 * // changer la valeur de la sortie
 * gpio->changerValeur(true);	// mise à 1 de la sortie
 *
 * // Avant d'appeller le destructeur il est possible de "suprimer" le port : 
 * // cette méthode à pour effet de suprimer le répertoire lié à "numPort" existant dans /sys/classe/gpio.
 * // Si cette méthode est appellée, la valeur matérielle du Gpio du raspberry est ramenée à la valeur par défaut.
 * // Dans le cas contraire, le port reste dans l'état écrit par les méthodes "configure" et "changerValeur"
 * gpio->supprimerPort();
 * // destruction de l'objet
 * delete gpio();
 * 
 * 
 *                    GNU GENERAL PUBLIC LICENSE
 *                       Version 3, 29 June 2007
 * 
 * 
 *    Copyright (C) 2015  françois GUILLET
 * 
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 * 
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <fstream>
#include <sstream>
using namespace std;

#ifndef Gpio_h
#define Gpio_h

class Gpio
{
  // numéro du port
  int numPort;

public :
  /// constructeur : 
  /// permet de choisir quel port sera utilisé
  /// le numéro de port doit être compris entre 1 et 28
  Gpio(int numeroPort);
  
  /// permet de choisir la direction d'utilisation du gpio (entrée ou sortie)
  /// il sera créé un dossier dans /sys/class/gpio/gpio{numeroPort}
  /// si utilisationEnEntree vaut false, cela indique que le port sera utilisé en sortie
  /// si utilisationEnEntree vaut true, cela indique que le port sera utilisé en entrée
  void configure(bool utilisationEnEntree);
  
  /// destructeur virtuel
  virtual ~Gpio();
  
  /// attention, le fait d'appeler cette méthode supprime le
  /// dossier créé par la méthode configure, et cela ramène la valeur matérielle par défaut du gpio
  void supprimerPort();

  /// permet d'écrire la valeur en sortie
  /// si value vaut true, la valeur du port matériel sera mise à 3.3V
  /// si value vaut false, la valeur du port matériel sera mise à 0V
  void changerValeur(bool value);
  
  /// permet de lire la valeur en sortie
  /// si value vaut true, cela signifie que la valeur du port matériel est connectée à 3.3V
  /// si value vaut false, cela signifie que la valeur du port matériel est connectée à 0V
  bool valeurSortie();
  
};

#endif
